# frozen_string_literal: true

require 'dry/logic/operations/and'
require 'dry/logic/operations/or'
require 'dry/logic/operations/xor'
require 'dry/logic/operations/implication'
require 'dry/logic/operations/negation'

require 'dry/logic/operations/key'
require 'dry/logic/operations/attr'
require 'dry/logic/operations/each'
require 'dry/logic/operations/set'
require 'dry/logic/operations/check'

require 'dry/logic/operators'
