# frozen_string_literal: true

# A collection of micro-libraries, each intended to encapsulate
# a common task in Ruby
module Dry
  module Logic
  end
end

require 'dry/logic/rule/predicate'
require 'dry/logic/operations'
